#include <Application.h>
#include <Window.h>
#include <Message.h>
#include <Bitmap.h>
#include <View.h>
#include <Rect.h>
#include <SupportDefs.h>

#include "Theme.h"
#include "Label.h"

BTLabel::BTLabel(BRect rect, const char *Name, const char *Label,
				BBitmap *background,uint32 _style,uint32 resize)
:BView(rect, Name, resize, B_FRAME_EVENTS) //B_WILL_DRAW
{
	fBackGround=background;
	fName=Name;
	fLabel.SetTo(Label);
	style=_style;
	
	BFont font = be_plain_font;
	font.SetSize(10);
	SetFont(&font);	
	
	text_color=HighColor();

	OVbmp=new BBitmap(BRect(0,0,rect.Width(),rect.Height()),B_RGBA32_BIG,true);
	OVview=new BView(BRect(0,0,rect.Width(),rect.Height()),"",0,0);

}

BTLabel::~BTLabel()
{
	delete OVbmp;
}

void BTLabel::SetTextColor(rgb_color c)
{
	text_color=c;
}

void BTLabel::SetText(const char *label)
{
	fLabel.SetTo(label);
	MyDraw();
}

void BTLabel::SetBackground(BBitmap *background)
{
	fBackGround=background;
	MyDraw();
}

void BTLabel::MyDraw(void)
{	
   OVview->LockLooper();
   OVview->SetDrawingMode(B_OP_COPY);
   
   if(fBackGround!=NULL)
	{
	BRect r=fBackGround->Bounds();	
	float background_dx=(int)Frame().left%(int)(fBackGround->Bounds().Width()+1);
	float background_dy=(int)Frame().top%(int)(fBackGround->Bounds().Height()+1);
	
	for(float y=0;y<Bounds().Height()+r.Height();y+=r.Height()+1)
		for(float x=0;x<Bounds().Width()+r.Width();x+=r.Width()+1)
		{
			r.OffsetTo(x-background_dx,y-background_dy);
			OVview->DrawBitmapAsync(fBackGround,r);
		}
	}
	else
	{
		OVview->SetHighColor(ViewColor());
		OVview->FillRect(Bounds());			
	}

	BFont font;
	GetFont(&font);
	OVview->SetFont(&font);

	float text_width=StringWidth(fLabel.String());
	float text_height=10;
	
	text_dx=(Bounds().Width()-text_width)/2.0;
	text_dy=((Bounds().Height()-text_height)/2.0)+1;
	
	if(style&BL_RECT_FLAG==BL_RECT_FLAG)
	{
		OVview->SetHighColor(128,128,128);
		OVview->StrokeRect(Bounds());	
	}
				
	OVview->SetDrawingMode(B_OP_ALPHA);
	OVview->SetHighColor(text_color.red,text_color.green,text_color.blue,250);
	OVview->DrawString(fLabel.String(),BPoint(text_dx+1,Bounds().bottom-text_dy));
	
	OVview->Sync();
    OVview->UnlockLooper();
    
	SetViewBitmap(OVbmp,Bounds(),Bounds(),B_FOLLOW_TOP | B_FOLLOW_RIGHT,0); 
	Invalidate(); 
}

void BTLabel::AttachedToWindow()
{
	//SetViewColor(Parent()->ViewColor());
	OVbmp->AddChild(OVview);
	MyDraw();
}

void
BTLabel::FrameResized(float width, float height)
{
	if(OVbmp!=NULL)
	{
	 OVbmp->RemoveChild(OVview);
	 OVbmp->~BBitmap();
 	 delete OVbmp;
	}
	
	OVbmp=new BBitmap(BRect(0,0,width,height),B_RGBA32_BIG,true);
	OVview->ResizeTo(width,height);
	OVbmp->AddChild(OVview);
	
	MyDraw();
}