#include <Application.h>
#include <Deskbar.h>
#include <Roster.h>
#include <Font.h>
#include <Alert.h>
#include <FilePanel.h>

#include "NaviDicWindow.h"
#include "Tip.h"
#include "Label.h"
#include "Theme.h"
#include "MainApp.h"

sem_id render_sem;	

BTip* ToolTip;

DjVuApp::DjVuApp(int argc, char **argv) : BApplication("application/x-vnd.DjVuViewer")
{
	fPulseStarted = false;
	fOpenPanel = new BFilePanel(B_OPEN_PANEL);
	//fOpenPanel->Window()->SetTitle("Open file...");
	if(SETTINGS->Load(SETTINGSFILE)==false)
		{
		SETTINGS->SetColor("CursBarCol",0,150,200);
		SETTINGS->SetColor("CursFrameCol",0,100,150);
		SETTINGS->SetColor("CursTextCol",255,255,255);
		SETTINGS->SetColor("EvenLinesCol",255,240,231);
		SETTINGS->SetColor("EvenTextCol",0,0,0);
		SETTINGS->SetColor("OddLinesCol",255,245,235);
		SETTINGS->SetColor("OddTextCol",0,0,0);
		SETTINGS->SetColor("ThumbBackCol",255,245,235);
		SETTINGS->SetColor("ThumbTextCol",0,0,0);
		

		SETTINGS->SetColor("ViewFrameCol",128,128,128);
		SETTINGS->SetColor("ViewDlgCol",255,245,235);
		SETTINGS->SetColor("TextNormCol",64,64,64);
	
		SETTINGS->SetFont("WLListFont","","",13,90);
		SETTINGS->SetFont("FVFont","","",12,90);
		SETTINGS->SetFont("TVNormalFont","","",12,90);

		SETTINGS->SetInt32("StorePosition",0);
		SETTINGS->SetInt32("ShowButtonLabel",1);
		
		SETTINGS->SetString("Theme","Default");
		SETTINGS->SetString("Language","English");

		SETTINGS->SetBool("Dither",false);
		SETTINGS->SetBool("ShrinkToBounds",false);
		SETTINGS->SetBool("ZoomToBounds",false);
		SETTINGS->SetBool("ScaleBilinear",true);
	
		SETTINGS->Save(SETTINGSFILE);
		}

	CurrentLanguage.SetDirectory("./Languages");
	CurrentLanguage.SetLanguage(SETTINGS->GetString("Language"));
	CurrentLanguage.SetDefaultLanguage("English");
	
	appTheme.SetDirectory("./Themes");
	appTheme.SetTheme(SETTINGS->GetString("Theme"));
	
	update_font_families(false);

	ToolTip=new BTip();
	if(ToolTip!=NULL)ToolTip->Enable(SETTINGS->GetInt32("UseToolTips",true));
	
	if((render_sem=create_sem(1,"render_sem"))<B_NO_ERROR)Quit();

}

DjVuApp::~DjVuApp()
{
 delete_sem(render_sem);
}

void DjVuApp::MessageReceived(BMessage *message)
{
	switch(message->what)
	{
	case MSG_FILE_OPEN:
			fOpenPanel->Show();
			break;

	case B_CANCEL:
			// File open panel was closed,
			// start checking count of open windows
			StartPulse();
			break;
	default:
		BApplication::MessageReceived(message);
		break;
	}
}

void
DjVuApp::StartPulse()
{
	if (!fPulseStarted) {
		// Tell the app to begin checking
		// for the number of open windows
		fPulseStarted = true;
		SetPulseRate(250000);
			// Set pulse to every 1/4 second
	}
}

void
DjVuApp::Pulse()
{
	// Bug: The BFilePanel is automatically closed if the volume that
	// is displayed is unmounted.
	if (!IsLaunching() && CountWindows() <= WINDOWS_TO_IGNORE)
		// If the application is not launching and
		// all windows are closed except for the file open panel,
		// quit the application
		PostMessage(B_QUIT_REQUESTED);
}

PSettings *
DjVuApp::Settings(void)
{
	return &ProgramSettings;
}
BTheme *
DjVuApp::Theme(void)
{
	return &appTheme;
}

BLanguage *
DjVuApp::Language(void)
{
	return &CurrentLanguage;
}

BPath
DjVuApp::Path(const char *dir)
{
	app_info inf;
   	be_app->GetAppInfo(&inf);
   	BPath bpath = BPath(&(inf.ref));
   	BPath par; 
   	bpath.GetParent(&par);
   	par.Append(dir);
	return(par);
}

void
DjVuApp::RefsReceived(BMessage *pmsg)
{
	if (pmsg->HasMessenger("TrackerViewToken")) {
		pmsg->FindMessenger("TrackerViewToken", &fTrackerMessenger);
	}

	uint32 type;
	int32 count;
	status_t ret = pmsg->GetInfo("refs", &type, &count);
	if (ret != B_OK || type != B_REF_TYPE)
		return;

	entry_ref ref;
	for (int32 i = 0; i < count; i++) {
   		if (pmsg->FindRef("refs", i, &ref) == B_OK)
   			OpenFile(&ref);
   	}
}


void
DjVuApp::ArgvReceived(int32 argc, char **argv)
{
	BMessage *pmsg = NULL;
	for (int32 i = 1; i < argc; i++) {
		entry_ref ref;
		status_t err = get_ref_for_path(argv[i], &ref);
		if (err == B_OK) {
			if (!pmsg) {
				pmsg = new BMessage;
				pmsg->what = B_REFS_RECEIVED;
			}
			pmsg->AddRef("refs", &ref);
		}
	}
	if (pmsg) {
		RefsReceived(pmsg);
		delete pmsg;
	}
}

void
DjVuApp::ReadyToRun()
{
	if (CountWindows() == WINDOWS_TO_IGNORE)
		fOpenPanel->Show();
	else
		StartPulse();
}

void
DjVuApp::OpenFile(const entry_ref *pref)
{
	DjVuWindow *window=new DjVuWindow("DjVu Viewer:");
	BPath path;
	BEntry entry=BEntry(pref);
	entry.GetPath(&path);
	window->LoadFile(path.Path());
	window->Show();
}

void
DjVuApp::Quit()
{
	BApplication::Quit();
}

int main(int argc, char **argv)
{
	DjVuApp application(argc,argv);
	
	application.Run();
	return 0;
}


