#include <stdlib.h>
#include <strings.h>
#include "Water.h"
#include <math.h>

#include "WaterRender.h"


#define random( min, max ) (( rand() % (int)((( max ) + 1 ) - ( min ))) + ( min ))

BWaterProc::BWaterProc()
{
	HeightField1 = NULL;
	HeightField2 = NULL;

	Width = 0;
	Height = 0;
	
	LightModifier = 1;
	Hpage = 0;
	m_density = 5;
}

BWaterProc::~BWaterProc()
{
	// Cleanup
	if(HeightField1 != NULL)
		delete [] HeightField1;
	if(HeightField2 != NULL)
		delete [] HeightField2;

	HeightField1 = NULL;
	HeightField2 = NULL;
}
void BWaterProc::Create(int iWidth,int iHeight)
{
	if(HeightField1 != NULL)
		delete [] HeightField1;
	if(HeightField2 != NULL)
		delete [] HeightField2;

	// Create our height fields
	HeightField1 = new int[(iWidth*iHeight)];
	HeightField2 = new int[(iWidth*iHeight)];

	// Clear our height fields
	memset(HeightField1,0,(iWidth*iHeight)*sizeof(int));
	memset(HeightField2,0,(iWidth*iHeight)*sizeof(int));

	Width = iWidth;
	Height = iHeight;

	// Set our page to 0
	Hpage = 0;

}
void BWaterProc::FlattenWater()
{
	memset(HeightField1,0,(Width*Height)*sizeof(int));
	memset(HeightField2,0,(Width*Height)*sizeof(int));
}

void BWaterProc::Render(DWORD* pSrcImage,DWORD* pTargetImage)
{
	DrawWaterWithLight(Hpage,LightModifier,pSrcImage,pTargetImage);
	CalcWater(Hpage,m_density);
	Hpage ^= 1;
}

void BWaterProc::CalcWater(int npage, int density)
{
  int newh;
  int count = Width + 1;
  int *newptr;
  int *oldptr;

  // Set up the pointers
  if(npage == 0)
  {
	newptr = &HeightField1[0];
	oldptr = &HeightField2[0];
  }
  else
  {
	newptr = &HeightField2[0];
	oldptr = &HeightField1[0];
  }

  int x, y;

  // Sorry, this function might not be as readable as I'd like, because
  // I optimized it somewhat.  (enough to make me feel satisfied with it)

  for (y = (Height-1)*Width; count < y; count += 2)
  {
    for (x = count+Width-2; count < x; count++)
    {

// This does the eight-pixel method.  It looks much better.

      newh          = ((oldptr[count + Width]
                      + oldptr[count - Width]
                      + oldptr[count + 1]
                      + oldptr[count - 1]
                      + oldptr[count - Width - 1]
                      + oldptr[count - Width + 1]
                      + oldptr[count + Width - 1]
                      + oldptr[count + Width + 1]
                       ) >> 2 )
                      - newptr[count];


      newptr[count] =  newh - (newh >> density);
    }
  }
}

void BWaterProc::HeightBlob(int x, int y, int radius, int height, int page)
{
  int rquad;
  int cx, cy, cyq;
  int left, top, right, bottom;

  int *newptr;
  int *oldptr;

  // Set up the pointers
  if(page == 0)
  {
	newptr = &HeightField1[0];
	oldptr = &HeightField2[0];
  }
  else
  {
	newptr = &HeightField2[0];
	oldptr = &HeightField1[0];
  }

  rquad = radius * radius;

  // Make a randomly-placed blob...
  if(x<0) x = 1+radius+ rand()%(Width-2*radius-1);
  if(y<0) y = 1+radius+ rand()%(Height-2*radius-1);

  left=-radius; right = radius;
  top=-radius; bottom = radius;

  // Perform edge clipping...
  if(x - radius < 1) left -= (x-radius-1);
  if(y - radius < 1) top  -= (y-radius-1);
  if(x + radius > Width-1) right -= (x+radius-Width+1);
  if(y + radius > Height-1) bottom-= (y+radius-Height+1);


  for(cy = top; cy < bottom; cy++)
  {
    cyq = cy*cy;
    for(cx = left; cx < right; cx++)
    {
      if(cx*cx + cyq < rquad)
        newptr[Width*(cy+y) + (cx+x)] += height;
    }
  }

}


void BWaterProc::DrawWaterWithLight(int page, int LightModifier,DWORD* pSrcImage,DWORD* pTargetImage)
{
  int dx, dy;
  int x, y;
  DWORD c;

  int offset=Width + 1;
  long lIndex;
  long lBreak = Width*Height;

  int *ptr = &HeightField1[0];


  for (y = (Height-1)*Width; offset < y; offset += 2)
  {
    for (x = offset+Width-2; offset < x; offset++)
    {
      dx = ptr[offset] - ptr[offset+1];
      dy = ptr[offset] - ptr[offset+Width];

	  lIndex = offset + Width*(dy>>3) + (dx>>3);
	  if(lIndex < lBreak && lIndex > 0)
	  {
		  c = pSrcImage[lIndex];// - (dx>>LightModifier);
		  // Now we shift it by the dx component...
		  // 
		  c = ::AsmGetShiftedColor(c,dx);

 		  pTargetImage[offset] = c;
 	  }

      offset++;
      dx = ptr[offset] - ptr[offset+1];
      dy = ptr[offset] - ptr[offset+Width];

	  lIndex = offset + Width*(dy>>3) + (dx>>3);
	  if(lIndex < lBreak && lIndex > 0)
	  {
		  c = pSrcImage[lIndex];// - (dx>>LightModifier);
		  c = ::AsmGetShiftedColor(c,dx);
		  pTargetImage[offset] = c;
	  }
 
    }
  }
 
}
/*
inline COLORREF BWaterProc::GetShiftedColor(COLORREF color,int shift)
{
	register long R;
	register long G;
	register long B;
	int ir;
	int ig;
	int ib;

	R = (color&255)-shift;
	G = ((color>>8)&255)-shift;
	B = ((color>>16)&255)-shift;

	ir = (R < 0) ? 0 : (R > 255) ? 255 : R;
	ig = (G < 0) ? 0 : (G > 255) ? 255 : G;
	ib = (B < 0) ? 0 : (B > 255) ? 255 : B;

	return ir+(ig<<8)+(ib<<16);
}
*/
